'use strict';

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

var _fieldKeys = require('../fieldKeys');

var _fieldKeys2 = _interopRequireDefault(_fieldKeys);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

describe('fieldKeys', function () {
  it('should return an empty array if no fields or values', function () {
    (0, _expect2.default)([].concat(_toConsumableArray((0, _fieldKeys2.default)()))).toEqual([]);
  });

  it('should handle simple fields with no values', function () {
    (0, _expect2.default)([].concat(_toConsumableArray((0, _fieldKeys2.default)(['a', 'b', 'c'])))).toEqual(['a', 'b', 'c']);
  });

  it('should handle simple fields with values', function () {
    (0, _expect2.default)([].concat(_toConsumableArray((0, _fieldKeys2.default)(['a', 'b', 'c'], { a: 1, b: 2, c: 3 })))).toEqual(['a', 'b', 'c']);
  });

  it('should handle deep fields with no values', function () {
    (0, _expect2.default)([].concat(_toConsumableArray((0, _fieldKeys2.default)(['a.b.c', 'd.e.f'])))).toEqual(['a.b.c', 'd.e.f']);
  });

  it('should handle deep fields with values', function () {
    (0, _expect2.default)([].concat(_toConsumableArray((0, _fieldKeys2.default)(['a.b.c', 'd.e.f'], {
      a: {
        b: {
          c: 42
        }
      },
      d: {
        e: {
          f: 43
        }
      }
    })))).toEqual(['a.b.c', 'd.e.f']);
  });

  it('should handle array fields with no values', function () {
    (0, _expect2.default)([].concat(_toConsumableArray((0, _fieldKeys2.default)(['a[]', 'b.c[]', 'd[].e'])))).toEqual(['a[]', 'b.c[]', 'd[]']);
  });

  it('should handle array fields with values', function () {
    (0, _expect2.default)([].concat(_toConsumableArray((0, _fieldKeys2.default)(['a[]', 'b.c[]', 'd[].e', 'f[].g'], {
      a: ['dog', 'cat'],
      b: {
        c: ['pig']
      },
      d: [{ e: 'h' }, { e: 'i' }]
    })))).toEqual(['a[0]', 'a[1]', 'b.c[0]', 'd[0].e', 'd[1].e', 'f[]']);
  });
});