'use strict';

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

var _flattenPaths = require('../flattenPaths');

var _flattenPaths2 = _interopRequireDefault(_flattenPaths);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('flattenPaths', function () {
  it('should return an empty map if no fields are provided', function () {
    (0, _expect2.default)((0, _flattenPaths2.default)(undefined)).toEqual({});
    (0, _expect2.default)((0, _flattenPaths2.default)(null)).toEqual({});
    (0, _expect2.default)((0, _flattenPaths2.default)(7)).toEqual({});
    (0, _expect2.default)((0, _flattenPaths2.default)({})).toEqual({});
    (0, _expect2.default)((0, _flattenPaths2.default)([])).toEqual({});
  });

  it('should flatten an already flat object', function () {
    (0, _expect2.default)((0, _flattenPaths2.default)({ a: 1, b: 2, c: 3 })).toEqual({ a: 1, b: 2, c: 3 });
  });

  it('should flatten an array into a flat object', function () {
    (0, _expect2.default)((0, _flattenPaths2.default)(['a', 'b', 'c'])).toEqual({ '0': 'a', '1': 'b', '2': 'c' });
  });

  it('should flatten an arbitrarily deep structure', function () {
    (0, _expect2.default)((0, _flattenPaths2.default)({
      a: {
        b: {
          c: 'the',
          d: 'quick'
        },
        c: 'brown'
      },
      d: ['fox', 'jumps', {
        e: 'over'
      }]
    })).toEqual({
      'a.b.c': 'the',
      'a.b.d': 'quick',
      'a.c': 'brown',
      'd[0]': 'fox',
      'd[1]': 'jumps',
      'd[2].e': 'over'
    });
  });
});