'use strict';

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

var _generateFields = require('../generateFields');

var _generateFields2 = _interopRequireDefault(_generateFields);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var createSpies = function createSpies() {
  return {
    createField: (0, _expect.createSpy)(function (key) {
      return { key: key };
    }).andCallThrough(),
    createArray: (0, _expect.createSpy)(function (key) {
      var initial = arguments.length <= 1 || arguments[1] === undefined ? [] : arguments[1];

      initial.key = key;
      return initial;
    }).andCallThrough()
  };
};

describe('generateFields', function () {
  it('should return an empty tree if no fields or values', function () {
    var _createSpies = createSpies();

    var createField = _createSpies.createField;
    var createArray = _createSpies.createArray;

    (0, _expect2.default)((0, _generateFields2.default)(undefined, undefined, undefined, createField, createArray)).toEqual({});
    (0, _expect2.default)(createField).toNotHaveBeenCalled();
    (0, _expect2.default)(createArray).toNotHaveBeenCalled();
  });

  it('should initialize simple fields with no values', function () {
    var _createSpies2 = createSpies();

    var createField = _createSpies2.createField;
    var createArray = _createSpies2.createArray;

    (0, _expect2.default)((0, _generateFields2.default)(['a', 'b', 'c'], undefined, undefined, createField, createArray)).toEqual({
      a: { key: 'a' },
      b: { key: 'b' },
      c: { key: 'c' }
    });
    (0, _expect2.default)(createField).toHaveBeenCalled();
    (0, _expect2.default)(createField.calls.length).toBe(3);
    (0, _expect2.default)(createField.calls[0].arguments).toEqual(['a']);
    (0, _expect2.default)(createField.calls[1].arguments).toEqual(['b']);
    (0, _expect2.default)(createField.calls[2].arguments).toEqual(['c']);
    (0, _expect2.default)(createArray).toNotHaveBeenCalled();
  });

  it('should initialize simple fields with values', function () {
    var _createSpies3 = createSpies();

    var createField = _createSpies3.createField;
    var createArray = _createSpies3.createArray;

    (0, _expect2.default)((0, _generateFields2.default)(['a', 'b', 'c'], undefined, {
      a: 'dog',
      b: 'cat',
      c: 'pig'
    }, createField, createArray)).toEqual({
      a: { key: 'a' },
      b: { key: 'b' },
      c: { key: 'c' }
    });
    (0, _expect2.default)(createField).toHaveBeenCalled();
    (0, _expect2.default)(createField.calls.length).toBe(3);
    (0, _expect2.default)(createField.calls[0].arguments).toEqual(['a']);
    (0, _expect2.default)(createField.calls[1].arguments).toEqual(['b']);
    (0, _expect2.default)(createField.calls[2].arguments).toEqual(['c']);
    (0, _expect2.default)(createArray).toNotHaveBeenCalled();
  });

  it('should not mutate simple fields with no changes', function () {
    var _createSpies4 = createSpies();

    var createField = _createSpies4.createField;
    var createArray = _createSpies4.createArray;

    var tree = {
      a: { key: 'a' },
      b: { key: 'b' },
      c: { key: 'c' }
    };
    var result = (0, _generateFields2.default)(['a', 'b', 'c'], tree, {
      a: 'dog',
      b: 'cat',
      c: 'pig'
    }, createField, createArray);
    (0, _expect2.default)(result).toBe(tree);
    (0, _expect2.default)(result.a).toBe(tree.a);
    (0, _expect2.default)(result.b).toBe(tree.b);
    (0, _expect2.default)(result.c).toBe(tree.c);
    (0, _expect2.default)(createField).toNotHaveBeenCalled();
    (0, _expect2.default)(createArray).toNotHaveBeenCalled();
  });

  it('should only mutate simple fields changed branch', function () {
    var _createSpies5 = createSpies();

    var createField = _createSpies5.createField;
    var createArray = _createSpies5.createArray;

    var tree = {
      a: { key: 'a' },
      b: { key: 'b' },
      c: { key: 'c' }
    };
    var result = (0, _generateFields2.default)(['a', 'b', 'c', 'd'], tree, {
      a: 'dog',
      b: 'cat',
      c: 'pig'
    }, createField, createArray);
    (0, _expect2.default)(result).toEqual({
      a: { key: 'a' },
      b: { key: 'b' },
      c: { key: 'c' },
      d: { key: 'd' }
    });
    (0, _expect2.default)(result).toNotBe(tree);
    (0, _expect2.default)(result.a).toBe(tree.a);
    (0, _expect2.default)(result.b).toBe(tree.b);
    (0, _expect2.default)(result.c).toBe(tree.c);
    (0, _expect2.default)(createField).toHaveBeenCalled();
    (0, _expect2.default)(createField.calls.length).toBe(1);
    (0, _expect2.default)(createField.calls[0].arguments).toEqual(['d']);
    (0, _expect2.default)(createArray).toNotHaveBeenCalled();
  });

  it('should initialize deep fields with no values', function () {
    var _createSpies6 = createSpies();

    var createField = _createSpies6.createField;
    var createArray = _createSpies6.createArray;

    (0, _expect2.default)((0, _generateFields2.default)(['deep.a', 'deep.b', 'other.c'], undefined, undefined, createField, createArray)).toEqual({
      deep: {
        a: { key: 'deep.a' },
        b: { key: 'deep.b' }
      },
      other: {
        c: { key: 'other.c' }
      }
    });
    (0, _expect2.default)(createField).toHaveBeenCalled();
    (0, _expect2.default)(createField.calls.length).toBe(3);
    (0, _expect2.default)(createField.calls[0].arguments).toEqual(['deep.a']);
    (0, _expect2.default)(createField.calls[1].arguments).toEqual(['deep.b']);
    (0, _expect2.default)(createField.calls[2].arguments).toEqual(['other.c']);
    (0, _expect2.default)(createArray).toNotHaveBeenCalled();
  });

  it('should initialize deep fields with values', function () {
    var _createSpies7 = createSpies();

    var createField = _createSpies7.createField;
    var createArray = _createSpies7.createArray;

    (0, _expect2.default)((0, _generateFields2.default)(['deep.a', 'deep.b', 'other.c'], undefined, {
      deep: {
        a: 'dog',
        b: 'cat'
      },
      other: {
        c: 'pig'
      }
    }, createField, createArray)).toEqual({
      deep: {
        a: { key: 'deep.a' },
        b: { key: 'deep.b' }
      },
      other: {
        c: { key: 'other.c' }
      }
    });
    (0, _expect2.default)(createField).toHaveBeenCalled();
    (0, _expect2.default)(createField.calls.length).toBe(3);
    (0, _expect2.default)(createField.calls[0].arguments).toEqual(['deep.a']);
    (0, _expect2.default)(createField.calls[1].arguments).toEqual(['deep.b']);
    (0, _expect2.default)(createField.calls[2].arguments).toEqual(['other.c']);
    (0, _expect2.default)(createArray).toNotHaveBeenCalled();
  });

  it('should not mutate deep fields with no changes', function () {
    var _createSpies8 = createSpies();

    var createField = _createSpies8.createField;
    var createArray = _createSpies8.createArray;

    var tree = {
      deep: {
        a: { key: 'deep.a' },
        b: { key: 'deep.b' }
      },
      other: {
        c: { key: 'other.c' }
      }
    };
    var result = (0, _generateFields2.default)(['deep.a', 'deep.b', 'other.c'], tree, {
      deep: {
        a: 'dog',
        b: 'cat'
      },
      other: {
        c: 'pig'
      }
    }, createField, createArray);
    (0, _expect2.default)(result).toBe(tree);
    (0, _expect2.default)(result.deep).toBe(tree.deep);
    (0, _expect2.default)(result.deep.a).toBe(tree.deep.a);
    (0, _expect2.default)(result.deep.b).toBe(tree.deep.b);
    (0, _expect2.default)(result.other).toBe(tree.other);
    (0, _expect2.default)(result.other.c).toBe(tree.other.c);
    (0, _expect2.default)(createField).toNotHaveBeenCalled();
    (0, _expect2.default)(createArray).toNotHaveBeenCalled();
  });

  it('should only mutate deep fields changed branch', function () {
    var _createSpies9 = createSpies();

    var createField = _createSpies9.createField;
    var createArray = _createSpies9.createArray;

    var tree = {
      deep: {
        a: { key: 'deep.a' },
        b: { key: 'deep.b' }
      },
      other: {
        c: { key: 'other.c' }
      }
    };
    var result = (0, _generateFields2.default)(['deep.a', 'deep.b', 'other.c', 'other.d'], tree, {
      deep: {
        a: 'dog',
        b: 'cat'
      },
      other: {
        c: 'pig'
      }
    }, createField, createArray);
    (0, _expect2.default)(result).toEqual({
      deep: {
        a: { key: 'deep.a' },
        b: { key: 'deep.b' }
      },
      other: {
        c: { key: 'other.c' },
        d: { key: 'other.d' }
      }
    });
    (0, _expect2.default)(result).toNotBe(tree);
    (0, _expect2.default)(result.deep).toBe(tree.deep);
    (0, _expect2.default)(result.deep.a).toBe(tree.deep.a);
    (0, _expect2.default)(result.deep.b).toBe(tree.deep.b);
    (0, _expect2.default)(result.other).toNotBe(tree.other);
    (0, _expect2.default)(result.other.c).toBe(tree.other.c);
    (0, _expect2.default)(createField).toHaveBeenCalled();
    (0, _expect2.default)(createField.calls.length).toBe(1);
    (0, _expect2.default)(createField.calls[0].arguments).toEqual(['other.d']);
    (0, _expect2.default)(createArray).toNotHaveBeenCalled();
  });

  it('should initialize array fields with no values', function () {
    var _createSpies10 = createSpies();

    var createField = _createSpies10.createField;
    var createArray = _createSpies10.createArray;

    (0, _expect2.default)((0, _generateFields2.default)(['a[]', 'b[].c', 'd.e[]'], undefined, undefined, createField, createArray)).toEqual({
      a: [],
      b: [],
      d: {
        e: []
      }
    });
    (0, _expect2.default)(createField).toNotHaveBeenCalled();
    (0, _expect2.default)(createArray).toHaveBeenCalled();
    (0, _expect2.default)(createArray.calls.length).toBe(3);
    (0, _expect2.default)(createArray.calls[0].arguments[0]).toBe('a');
    (0, _expect2.default)(createArray.calls[1].arguments[0]).toBe('b');
    (0, _expect2.default)(createArray.calls[2].arguments[0]).toBe('d.e');
  });

  it('should initialize array fields with values', function () {
    var _createSpies11 = createSpies();

    var createField = _createSpies11.createField;
    var createArray = _createSpies11.createArray;

    var result = (0, _generateFields2.default)(['a[]', 'b[].c', 'd.e[]'], undefined, {
      a: ['dog'],
      b: [{ c: 'rat' }, { c: 'hog' }],
      d: {
        e: ['cow', 'sow']
      }
    }, createField, createArray);
    (0, _expect2.default)(result).toEqual({
      a: [{ key: 'a[0]' }],
      b: [{ c: { key: 'b[0].c' } }, { c: { key: 'b[1].c' } }],
      d: {
        e: [{ key: 'd.e[0]' }, { key: 'd.e[1]' }]
      }
    });
    (0, _expect2.default)(createField).toHaveBeenCalled();
    (0, _expect2.default)(createField.calls.length).toBe(5);
    (0, _expect2.default)(createField.calls[0].arguments[0]).toBe('a[0]');
    (0, _expect2.default)(createField.calls[1].arguments[0]).toBe('b[0].c');
    (0, _expect2.default)(createField.calls[2].arguments[0]).toBe('b[1].c');
    (0, _expect2.default)(createField.calls[3].arguments[0]).toBe('d.e[0]');
    (0, _expect2.default)(createField.calls[4].arguments[0]).toBe('d.e[1]');
    (0, _expect2.default)(createArray).toHaveBeenCalled();
    (0, _expect2.default)(createArray.calls.length).toBe(3);
    (0, _expect2.default)(createArray.calls[0].arguments[0]).toBe('a');
    (0, _expect2.default)(createArray.calls[1].arguments[0]).toBe('b');
    (0, _expect2.default)(createArray.calls[2].arguments[0]).toBe('d.e');
  });

  it('should not mutate array fields with no changes', function () {
    var _createSpies12 = createSpies();

    var createField = _createSpies12.createField;
    var createArray = _createSpies12.createArray;

    var tree = {
      a: [{ key: 'a[0]' }],
      b: [{ c: { key: 'b[0].c' } }, { c: { key: 'b[1].c' } }],
      d: {
        e: [{ key: 'd.e[0]' }, { key: 'd.e[1]' }]
      }
    };
    tree.a.key = 'a';
    tree.b.key = 'b';
    tree.d.e.key = 'd.e';
    var result = (0, _generateFields2.default)(['a[]', 'b[].c', 'd.e[]'], tree, {
      a: ['dog'],
      b: [{ c: 'rat' }, { c: 'hog' }],
      d: {
        e: ['cow', 'sow']
      }
    }, createField, createArray);
    (0, _expect2.default)(result).toBe(tree);
    (0, _expect2.default)(result.a).toBe(tree.a);
    (0, _expect2.default)(result.a[0]).toBe(tree.a[0]);
    (0, _expect2.default)(result.b).toBe(tree.b);
    (0, _expect2.default)(result.b[0]).toBe(tree.b[0]);
    (0, _expect2.default)(result.b[0].c).toBe(tree.b[0].c);
    (0, _expect2.default)(result.b[1]).toBe(tree.b[1]);
    (0, _expect2.default)(result.b[1].c).toBe(tree.b[1].c);
    (0, _expect2.default)(result.d).toBe(tree.d);
    (0, _expect2.default)(result.d.e).toBe(tree.d.e);
    (0, _expect2.default)(result.d.e[0]).toBe(tree.d.e[0]);
    (0, _expect2.default)(result.d.e[1]).toBe(tree.d.e[1]);
    (0, _expect2.default)(createField).toNotHaveBeenCalled();
    (0, _expect2.default)(createArray).toNotHaveBeenCalled();
  });

  it('should only mutate array fields changed branch when elements added', function () {
    var _createSpies13 = createSpies();

    var createField = _createSpies13.createField;
    var createArray = _createSpies13.createArray;

    var tree = {
      a: [{ key: 'a[0]' }],
      b: [{ c: { key: 'b[0].c' } }, { c: { key: 'b[1].c' } }],
      d: {
        e: [{ key: 'd.e[0]' }, { key: 'd.e[1]' }]
      }
    };
    tree.a.key = 'a';
    tree.b.key = 'b';
    tree.d.e.key = 'd.e';
    var result = (0, _generateFields2.default)(['a[]', 'b[].c', 'd.e[]'], tree, {
      a: ['dog'],
      b: [{ c: 'rat' }, { c: 'hog' }, { c: 'ram' }],
      d: {
        e: ['cow', 'sow', 'ewe']
      }
    }, createField, createArray);
    (0, _expect2.default)(result).toNotBe(tree);
    (0, _expect2.default)(result).toEqual({
      a: [{ key: 'a[0]' }],
      b: [{ c: { key: 'b[0].c' } }, { c: { key: 'b[1].c' } }, { c: { key: 'b[2].c' } }],
      d: {
        e: [{ key: 'd.e[0]' }, { key: 'd.e[1]' }, { key: 'd.e[2]' }]
      }
    });
    (0, _expect2.default)(result.a).toBe(tree.a);
    (0, _expect2.default)(result.a[0]).toBe(tree.a[0]);
    (0, _expect2.default)(result.b).toNotBe(tree.b);
    (0, _expect2.default)(result.b[0]).toBe(tree.b[0]);
    (0, _expect2.default)(result.b[0].c).toBe(tree.b[0].c);
    (0, _expect2.default)(result.b[1]).toBe(tree.b[1]);
    (0, _expect2.default)(result.b[1].c).toBe(tree.b[1].c);
    (0, _expect2.default)(result.d).toNotBe(tree.d);
    (0, _expect2.default)(result.d.e).toNotBe(tree.d.e);
    (0, _expect2.default)(result.d.e[0]).toBe(tree.d.e[0]);
    (0, _expect2.default)(result.d.e[1]).toBe(tree.d.e[1]);
    (0, _expect2.default)(createField).toHaveBeenCalled();
    (0, _expect2.default)(createField.calls.length).toBe(2);
    (0, _expect2.default)(createField.calls[0].arguments).toEqual(['b[2].c']);
    (0, _expect2.default)(createField.calls[1].arguments).toEqual(['d.e[2]']);
    (0, _expect2.default)(createArray).toHaveBeenCalled();
    (0, _expect2.default)(createArray.calls.length).toBe(2);
    (0, _expect2.default)(createArray.calls[0].arguments[0]).toBe('b');
    (0, _expect2.default)(createArray.calls[1].arguments[0]).toBe('d.e');
  });

  it('should only mutate array fields changed branch when elements removed', function () {
    var _createSpies14 = createSpies();

    var createField = _createSpies14.createField;
    var createArray = _createSpies14.createArray;

    var tree = {
      a: [{ key: 'a[0]' }],
      b: [{ c: { key: 'b[0].c' } }, { c: { key: 'b[1].c' } }],
      d: {
        e: [{ key: 'd.e[0]' }, { key: 'd.e[1]' }]
      }
    };
    tree.a.key = 'a';
    tree.b.key = 'b';
    tree.d.e.key = 'd.e';
    var result = (0, _generateFields2.default)(['a[]', 'b[].c', 'd.e[]'], tree, {
      a: ['dog'],
      b: [{ c: 'rat' }],
      d: {
        e: ['cow']
      }
    }, createField, createArray);
    (0, _expect2.default)(result).toNotBe(tree);
    (0, _expect2.default)(result).toEqual({
      a: [{ key: 'a[0]' }],
      b: [{ c: { key: 'b[0].c' } }],
      d: {
        e: [{ key: 'd.e[0]' }]
      }
    });
    (0, _expect2.default)(result.a).toBe(tree.a);
    (0, _expect2.default)(result.a[0]).toBe(tree.a[0]);
    (0, _expect2.default)(result.b).toNotBe(tree.b);
    (0, _expect2.default)(result.b[0]).toBe(tree.b[0]);
    (0, _expect2.default)(result.b[0].c).toBe(tree.b[0].c);
    (0, _expect2.default)(result.d).toNotBe(tree.d);
    (0, _expect2.default)(result.d.e).toNotBe(tree.d.e);
    (0, _expect2.default)(result.d.e[0]).toBe(tree.d.e[0]);
    (0, _expect2.default)(createField).toNotHaveBeenCalled();
    (0, _expect2.default)(createArray).toHaveBeenCalled();
    (0, _expect2.default)(createArray.calls.length).toBe(2);
    (0, _expect2.default)(createArray.calls[0].arguments[0]).toBe('b');
    (0, _expect2.default)(createArray.calls[1].arguments[0]).toBe('d.e');
  });
});