'use strict';

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

var _isPromise = require('is-promise');

var _isPromise2 = _interopRequireDefault(_isPromise);

var _handleSubmit = require('../handleSubmit');

var _handleSubmit2 = _interopRequireDefault(_handleSubmit);

var _SubmissionError = require('../SubmissionError');

var _SubmissionError2 = _interopRequireDefault(_SubmissionError);

var _lodash = require('lodash.noop');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('handleSubmit', function () {
  it('should stop if sync validation fails', function () {
    var values = { foo: 'bar', baz: 42 };
    var submit = (0, _expect.createSpy)().andReturn(69);
    var startSubmit = (0, _expect.createSpy)();
    var stopSubmit = (0, _expect.createSpy)();
    var submitFailed = (0, _expect.createSpy)();
    var asyncValidate = (0, _expect.createSpy)();
    var props = { startSubmit: startSubmit, stopSubmit: stopSubmit, submitFailed: submitFailed, values: values };

    (0, _handleSubmit2.default)(submit, props, false, asyncValidate);

    (0, _expect2.default)(submit).toNotHaveBeenCalled();
    (0, _expect2.default)(startSubmit).toNotHaveBeenCalled();
    (0, _expect2.default)(stopSubmit).toNotHaveBeenCalled();
    (0, _expect2.default)(asyncValidate).toNotHaveBeenCalled();
    (0, _expect2.default)(submitFailed).toHaveBeenCalled();
  });

  it('should stop and return rejected promise if sync validation fails and returnRejectedSubmitPromise', function (done) {
    var values = { foo: 'bar', baz: 42 };
    var submit = (0, _expect.createSpy)().andReturn(69);
    var syncErrors = { foo: 'error' };
    var startSubmit = (0, _expect.createSpy)();
    var stopSubmit = (0, _expect.createSpy)();
    var submitFailed = (0, _expect.createSpy)();
    var asyncValidate = (0, _expect.createSpy)();
    var props = {
      returnRejectedSubmitPromise: true,
      startSubmit: startSubmit,
      stopSubmit: stopSubmit,
      submitFailed: submitFailed,
      syncErrors: syncErrors,
      values: values
    };

    var result = (0, _handleSubmit2.default)(submit, props, false, asyncValidate);
    (0, _expect2.default)((0, _isPromise2.default)(result)).toBe(true);

    (0, _expect2.default)(asyncValidate).toNotHaveBeenCalled();
    (0, _expect2.default)(submit).toNotHaveBeenCalled();
    (0, _expect2.default)(startSubmit).toNotHaveBeenCalled();
    (0, _expect2.default)(stopSubmit).toNotHaveBeenCalled();
    (0, _expect2.default)(submitFailed).toHaveBeenCalled();
    result.then(function () {
      (0, _expect2.default)(false).toBe(true); // should not be in resolve branch
    }, function (error) {
      (0, _expect2.default)(error).toBe(syncErrors);
      done();
    });
  });

  it('should return result of sync submit', function () {
    var values = { foo: 'bar', baz: 42 };
    var submit = (0, _expect.createSpy)().andReturn(69);
    var dispatch = _lodash2.default;
    var startSubmit = (0, _expect.createSpy)();
    var stopSubmit = (0, _expect.createSpy)();
    var submitFailed = (0, _expect.createSpy)();
    var asyncValidate = undefined;
    var props = { dispatch: dispatch, startSubmit: startSubmit, stopSubmit: stopSubmit, submitFailed: submitFailed, values: values };

    (0, _expect2.default)((0, _handleSubmit2.default)(submit, props, true, asyncValidate)).toBe(69);

    (0, _expect2.default)(submit).toHaveBeenCalled().toHaveBeenCalledWith(values, dispatch);
    (0, _expect2.default)(startSubmit).toNotHaveBeenCalled();
    (0, _expect2.default)(stopSubmit).toNotHaveBeenCalled();
    (0, _expect2.default)(submitFailed).toNotHaveBeenCalled();
  });

  it('should not submit if async validation fails', function () {
    var values = { foo: 'bar', baz: 42 };
    var submit = (0, _expect.createSpy)().andReturn(69);
    var dispatch = _lodash2.default;
    var startSubmit = (0, _expect.createSpy)();
    var stopSubmit = (0, _expect.createSpy)();
    var submitFailed = (0, _expect.createSpy)();
    var asyncValidate = (0, _expect.createSpy)().andReturn(Promise.reject());
    var props = { dispatch: dispatch, startSubmit: startSubmit, stopSubmit: stopSubmit, submitFailed: submitFailed, values: values };

    return (0, _handleSubmit2.default)(submit, props, true, asyncValidate).then(function (result) {
      (0, _expect2.default)(result).toBe(undefined);
      (0, _expect2.default)(asyncValidate).toHaveBeenCalled().toHaveBeenCalledWith();
      (0, _expect2.default)(submit).toNotHaveBeenCalled();
      (0, _expect2.default)(startSubmit).toNotHaveBeenCalled();
      (0, _expect2.default)(stopSubmit).toNotHaveBeenCalled();
      (0, _expect2.default)(submitFailed).toHaveBeenCalled();
    }, function () {
      (0, _expect2.default)(false).toBe(true); // should not get into reject branch
    });
  });

  it('should not submit if async validation fails and return rejected promise', function () {
    var values = { foo: 'bar', baz: 42 };
    var submit = (0, _expect.createSpy)().andReturn(69);
    var dispatch = _lodash2.default;
    var startSubmit = (0, _expect.createSpy)();
    var stopSubmit = (0, _expect.createSpy)();
    var submitFailed = (0, _expect.createSpy)();
    var asyncErrors = { foo: 'async error' };
    var asyncValidate = (0, _expect.createSpy)().andReturn(Promise.reject(asyncErrors));
    var props = {
      dispatch: dispatch, startSubmit: startSubmit, stopSubmit: stopSubmit, submitFailed: submitFailed, values: values,
      returnRejectedSubmitPromise: true
    };

    return (0, _handleSubmit2.default)(submit, props, true, asyncValidate).then(function () {
      (0, _expect2.default)(false).toBe(true); // should not get into resolve branch
    }, function (result) {
      (0, _expect2.default)(result).toBe(asyncErrors);
      (0, _expect2.default)(asyncValidate).toHaveBeenCalled().toHaveBeenCalledWith();
      (0, _expect2.default)(submit).toNotHaveBeenCalled();
      (0, _expect2.default)(startSubmit).toNotHaveBeenCalled();
      (0, _expect2.default)(stopSubmit).toNotHaveBeenCalled();
      (0, _expect2.default)(submitFailed).toHaveBeenCalled();
    });
  });

  it('should sync submit if async validation passes', function () {
    var values = { foo: 'bar', baz: 42 };
    var submit = (0, _expect.createSpy)().andReturn(69);
    var dispatch = _lodash2.default;
    var startSubmit = (0, _expect.createSpy)();
    var stopSubmit = (0, _expect.createSpy)();
    var submitFailed = (0, _expect.createSpy)();
    var asyncValidate = (0, _expect.createSpy)().andReturn(Promise.resolve());
    var props = { dispatch: dispatch, startSubmit: startSubmit, stopSubmit: stopSubmit, submitFailed: submitFailed, values: values };

    return (0, _handleSubmit2.default)(submit, props, true, asyncValidate).then(function (result) {
      (0, _expect2.default)(result).toBe(69);
      (0, _expect2.default)(asyncValidate).toHaveBeenCalled().toHaveBeenCalledWith();
      (0, _expect2.default)(submit).toHaveBeenCalled().toHaveBeenCalledWith(values, dispatch);
      (0, _expect2.default)(startSubmit).toNotHaveBeenCalled();
      (0, _expect2.default)(stopSubmit).toNotHaveBeenCalled();
      (0, _expect2.default)(submitFailed).toNotHaveBeenCalled();
    }, function () {
      (0, _expect2.default)(false).toBe(true); // should not get into reject branch
    });
  });

  it('should async submit if async validation passes', function () {
    var values = { foo: 'bar', baz: 42 };
    var submit = (0, _expect.createSpy)().andReturn(Promise.resolve(69));
    var dispatch = _lodash2.default;
    var startSubmit = (0, _expect.createSpy)();
    var stopSubmit = (0, _expect.createSpy)();
    var submitFailed = (0, _expect.createSpy)();
    var asyncValidate = (0, _expect.createSpy)().andReturn(Promise.resolve());
    var props = { dispatch: dispatch, startSubmit: startSubmit, stopSubmit: stopSubmit, submitFailed: submitFailed, values: values };

    return (0, _handleSubmit2.default)(submit, props, true, asyncValidate).then(function (result) {
      (0, _expect2.default)(result).toBe(69);
      (0, _expect2.default)(asyncValidate).toHaveBeenCalled().toHaveBeenCalledWith();
      (0, _expect2.default)(submit).toHaveBeenCalled().toHaveBeenCalledWith(values, dispatch);
      (0, _expect2.default)(startSubmit).toHaveBeenCalled();
      (0, _expect2.default)(stopSubmit).toHaveBeenCalled().toHaveBeenCalledWith();
      (0, _expect2.default)(submitFailed).toNotHaveBeenCalled();
    }, function () {
      (0, _expect2.default)(false).toBe(true); // should not get into reject branch
    });
  });

  it('should set submit errors if async submit fails', function () {
    var values = { foo: 'bar', baz: 42 };
    var submitErrors = { foo: 'submit error' };
    var submit = (0, _expect.createSpy)().andReturn(Promise.reject(new _SubmissionError2.default(submitErrors)));
    var dispatch = _lodash2.default;
    var startSubmit = (0, _expect.createSpy)();
    var stopSubmit = (0, _expect.createSpy)();
    var submitFailed = (0, _expect.createSpy)();
    var asyncValidate = (0, _expect.createSpy)().andReturn(Promise.resolve());
    var props = { dispatch: dispatch, startSubmit: startSubmit, stopSubmit: stopSubmit, submitFailed: submitFailed, values: values };

    return (0, _handleSubmit2.default)(submit, props, true, asyncValidate).then(function (result) {
      (0, _expect2.default)(result).toBe(undefined);
      (0, _expect2.default)(asyncValidate).toHaveBeenCalled().toHaveBeenCalledWith();
      (0, _expect2.default)(submit).toHaveBeenCalled().toHaveBeenCalledWith(values, dispatch);
      (0, _expect2.default)(startSubmit).toHaveBeenCalled();
      (0, _expect2.default)(stopSubmit).toHaveBeenCalled().toHaveBeenCalledWith(submitErrors);
      (0, _expect2.default)(submitFailed).toNotHaveBeenCalled();
    }, function () {
      (0, _expect2.default)(false).toBe(true); // should not get into reject branch
    });
  });

  it('should set submit errors if async submit fails and return rejected promise', function () {
    var values = { foo: 'bar', baz: 42 };
    var submitErrors = { foo: 'submit error' };
    var submit = (0, _expect.createSpy)().andReturn(Promise.reject(new _SubmissionError2.default(submitErrors)));
    var dispatch = function dispatch() {
      return null;
    };
    var startSubmit = (0, _expect.createSpy)();
    var stopSubmit = (0, _expect.createSpy)();
    var submitFailed = (0, _expect.createSpy)();
    var asyncValidate = (0, _expect.createSpy)().andReturn(Promise.resolve());
    var props = {
      dispatch: dispatch, startSubmit: startSubmit, stopSubmit: stopSubmit, submitFailed: submitFailed, values: values,
      returnRejectedSubmitPromise: true
    };

    return (0, _handleSubmit2.default)(submit, props, true, asyncValidate).then(function () {
      (0, _expect2.default)(false).toBe(true); // should not get into resolve branch
    }, function (result) {
      (0, _expect2.default)(result instanceof _SubmissionError2.default).toBe(true);
      (0, _expect2.default)(result.errors).toBe(submitErrors);
      (0, _expect2.default)(asyncValidate).toHaveBeenCalled().toHaveBeenCalledWith();
      (0, _expect2.default)(submit).toHaveBeenCalled().toHaveBeenCalledWith(values, dispatch);
      (0, _expect2.default)(startSubmit).toHaveBeenCalled();
      (0, _expect2.default)(stopSubmit).toHaveBeenCalled().toHaveBeenCalledWith(submitErrors);
      (0, _expect2.default)(submitFailed).toNotHaveBeenCalled();
    });
  });
});