'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _actions = require('../actions');

var describeBlur = function describeBlur(reducer, expect, _ref) {
  var fromJS = _ref.fromJS;
  var setIn = _ref.setIn;
  return function () {
    it('should set value on blur with empty state', function () {
      var state = reducer(undefined, _extends({}, (0, _actions.blur)('myValue'), {
        form: 'foo',
        field: 'myField'
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            myField: 'myValue'
          }
        }
      });
    });

    it('should set value on blur and touch with empty state', function () {
      var state = reducer(undefined, _extends({}, (0, _actions.blur)('myValue'), {
        form: 'foo',
        field: 'myField',
        touch: true
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            myField: 'myValue'
          },
          fields: {
            myField: {
              touched: true
            }
          }
        }
      });
    });

    it('should set value on blur and touch with initial value', function () {
      var state = reducer(fromJS({
        foo: {
          values: {
            myField: 'initialValue'
          },
          initial: {
            myField: 'initialValue'
          },
          fields: {
            myField: {
              active: true
            }
          },
          active: 'myField'
        }
      }), _extends({}, (0, _actions.blur)('myValue'), {
        form: 'foo',
        field: 'myField',
        touch: true
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            myField: 'myValue'
          },
          initial: {
            myField: 'initialValue'
          },
          fields: {
            myField: {
              touched: true
            }
          }
        }
      });
    });

    it('should not modify value if undefined is passed on blur (for android react native)', function () {
      var state = reducer(fromJS({
        foo: {
          values: {
            myField: 'myValue'
          },
          initial: {
            myField: 'initialValue'
          },
          fields: {
            myField: {
              active: true
            }
          },
          active: 'myField'
        }
      }), _extends({}, (0, _actions.blur)(), {
        form: 'foo',
        field: 'myField',
        touch: true
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            myField: 'myValue'
          },
          initial: {
            myField: 'initialValue'
          },
          fields: {
            myField: {
              touched: true
            }
          }
        }
      });
    });

    it('should not modify value if undefined is passed on blur, even if no value existed (for android react native)', function () {
      var state = reducer(fromJS({
        foo: {
          fields: {
            myField: {
              active: true
            }
          },
          active: 'myField'
        }
      }), _extends({}, (0, _actions.blur)(), {
        form: 'foo',
        field: 'myField',
        touch: true
      }));
      expect(state).toEqualMap({
        foo: {
          fields: {
            myField: {
              touched: true
            }
          }
        }
      });
    });

    it('should remove a value if on blur is set with \'\'', function () {
      var state = reducer(fromJS({
        foo: {
          values: {
            myField: 'initialValue'
          }
        }
      }), _extends({}, (0, _actions.blur)(''), {
        form: 'foo',
        field: 'myField',
        touch: true
      }));
      expect(state).toEqualMap({
        foo: {
          fields: {
            myField: {
              touched: true
            }
          }
        }
      });
    });

    it('should remove nested value container if on blur clears all values', function () {
      var state = reducer(fromJS({
        foo: {
          values: {
            nested: {
              myField: 'initialValue'
            }
          }
        }
      }), _extends({}, (0, _actions.blur)(''), {
        form: 'foo',
        field: 'nested.myField',
        touch: true
      }));
      expect(state).toEqualMap({
        foo: {
          fields: {
            nested: {
              myField: {
                touched: true
              }
            }
          }
        }
      });
    });

    it('should set nested value on blur', function () {
      var state = reducer(fromJS({
        foo: {
          fields: {
            myField: {
              mySubField: {
                active: true
              }
            }
          },
          active: 'myField.mySubField'
        }
      }), _extends({}, (0, _actions.blur)('hello'), {
        form: 'foo',
        field: 'myField.mySubField',
        touch: true
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            myField: {
              mySubField: 'hello'
            }
          },
          fields: {
            myField: {
              mySubField: {
                touched: true
              }
            }
          }
        }
      });
    });

    it('should set array value on blur', function () {
      var state = reducer(fromJS({
        foo: {
          values: {
            myArray: []
          },
          fields: {
            myArray: [{ active: true }]
          },
          active: 'myArray[0]'
        }
      }), _extends({}, (0, _actions.blur)('hello'), {
        form: 'foo',
        field: 'myArray[0]',
        touch: true
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            myArray: ['hello']
          },
          fields: {
            myArray: [{
              touched: true
            }]
          }
        }
      });
    });

    it('should set complex value on blur', function () {
      var state = reducer(fromJS({
        foo: {
          fields: {
            myComplexField: {
              active: true
            }
          },
          active: 'myComplexField'
        }
      }), _extends({}, (0, _actions.blur)({ id: 42, name: 'Bobby' }), {
        form: 'foo',
        field: 'myComplexField',
        touch: true
      }));
      expect(state).toEqualMap(setIn(fromJS({ // must use setIn to make sure complex value is js object
        foo: {
          fields: {
            myComplexField: {
              touched: true
            }
          }
        }
      }), 'foo.values.myComplexField', { id: 42, name: 'Bobby' }));
    });
  };
};

exports.default = describeBlur;