'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _actions = require('../actions');

var describeChange = function describeChange(reducer, expect, _ref) {
  var fromJS = _ref.fromJS;
  return function () {
    it('should set value on change with empty state', function () {
      var state = reducer(undefined, _extends({}, (0, _actions.change)('myValue'), {
        form: 'foo',
        field: 'myField'
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            myField: 'myValue'
          }
        }
      });
    });

    it('should set value on change and touch with empty state', function () {
      var state = reducer(undefined, _extends({}, (0, _actions.change)('myValue'), {
        form: 'foo',
        field: 'myField',
        touch: true
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            myField: 'myValue'
          },
          fields: {
            myField: {
              touched: true
            }
          }
        }
      });
    });

    it('should set value on change and touch with initial value', function () {
      var state = reducer(fromJS({
        foo: {
          values: {
            myField: 'initialValue'
          },
          initial: {
            myField: 'initialValue'
          }
        }
      }), _extends({}, (0, _actions.change)('myValue'), {
        form: 'foo',
        field: 'myField',
        touch: true
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            myField: 'myValue'
          },
          initial: {
            myField: 'initialValue'
          },
          fields: {
            myField: {
              touched: true
            }
          }
        }
      });
    });

    it('should remove a value if on change is set with \'\'', function () {
      var state = reducer(fromJS({
        foo: {
          values: {
            myField: 'initialValue'
          }
        }
      }), _extends({}, (0, _actions.change)(''), {
        form: 'foo',
        field: 'myField',
        touch: true
      }));
      expect(state).toEqualMap({
        foo: {
          fields: {
            myField: {
              touched: true
            }
          }
        }
      });
    });

    it('should remove nested value container if on change clears all values', function () {
      var state = reducer(fromJS({
        foo: {
          values: {
            nested: {
              myField: 'initialValue'
            }
          }
        }
      }), _extends({}, (0, _actions.change)(''), {
        form: 'foo',
        field: 'nested.myField',
        touch: true
      }));
      expect(state).toEqualMap({
        foo: {
          fields: {
            nested: {
              myField: {
                touched: true
              }
            }
          }
        }
      });
    });

    it('should not modify a value if called with undefined', function () {
      var state = reducer(fromJS({
        foo: {
          values: {
            myField: 'initialValue'
          }
        }
      }), _extends({}, (0, _actions.change)(undefined), {
        form: 'foo',
        field: 'myField',
        touch: true
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            myField: 'initialValue'
          },
          fields: {
            myField: {
              touched: true
            }
          }
        }
      });
    });

    it('should set value on change and remove field-level submit and async errors', function () {
      var state = reducer(fromJS({
        foo: {
          values: {
            myField: 'initial'
          },
          asyncErrors: {
            myField: 'async error'
          },
          submitErrors: {
            myField: 'submit error'
          },
          error: 'some global error'
        }
      }), _extends({}, (0, _actions.change)('different'), {
        form: 'foo',
        field: 'myField'
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            myField: 'different'
          },
          error: 'some global error' // unaffected by CHANGE
        }
      });
    });

    it('should set nested value on change with empty state', function () {
      var state = reducer(undefined, _extends({}, (0, _actions.change)('myValue'), {
        form: 'foo',
        field: 'myField.mySubField'
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            myField: {
              mySubField: 'myValue'
            }
          }
        }
      });
    });
  };
};

exports.default = describeChange;