'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _actions = require('../actions');

var describeFocus = function describeFocus(reducer, expect, _ref) {
  var fromJS = _ref.fromJS;
  return function () {
    it('should set visited on focus and update active with no previous state', function () {
      var state = reducer(undefined, _extends({}, (0, _actions.focus)(), {
        form: 'foo',
        field: 'myField'
      }));
      expect(state).toEqualMap({
        foo: {
          fields: {
            myField: {
              visited: true,
              active: true
            }
          },
          active: 'myField'
        }
      });
    });

    it('should set visited on focus and update active on deep field with no previous state', function () {
      var state = reducer(undefined, _extends({}, (0, _actions.focus)(), {
        form: 'foo',
        field: 'myField.subField'
      }));
      expect(state).toEqualMap({
        foo: {
          fields: {
            myField: {
              subField: {
                visited: true,
                active: true
              }
            }
          },
          active: 'myField.subField'
        }
      });
    });

    it('should set visited on focus and update current with previous state', function () {
      var state = reducer(fromJS({
        foo: {
          values: {
            myField: 'initialValue'
          },
          initial: {
            myField: 'initialValue'
          },
          fields: {
            myField: {},
            otherField: {
              visited: true,
              active: true
            }
          },
          active: 'otherField'
        }
      }), _extends({}, (0, _actions.focus)(), {
        form: 'foo',
        field: 'myField'
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            myField: 'initialValue'
          },
          initial: {
            myField: 'initialValue'
          },
          fields: {
            otherField: {
              visited: true
            },
            myField: {
              visited: true,
              active: true
            }
          },
          active: 'myField'
        }
      });
    });
  };
};

exports.default = describeFocus;