'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _actions = require('../actions');

var describeUntouch = function describeUntouch(reducer, expect, _ref) {
  var fromJS = _ref.fromJS;
  return function () {
    it('should unmark fields as touched on untouch', function () {
      var state = reducer(fromJS({
        foo: {
          values: {
            myField: 'value',
            myOtherField: 'otherValue'
          },
          fields: {
            myField: {
              touched: true
            },
            myOtherField: {
              touched: true
            }
          }
        }
      }), _extends({}, (0, _actions.untouch)('myField', 'myOtherField'), {
        form: 'foo'
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            myField: 'value',
            myOtherField: 'otherValue'
          },
          fields: {
            myField: {},
            myOtherField: {}
          }
        }
      });
    });

    it('should unmark deep fields as touched on untouch', function () {
      var state = reducer(fromJS({
        foo: {
          values: {
            deep: {
              myField: 'value',
              myOtherField: 'otherValue'
            }
          },
          fields: {
            deep: {
              myField: {
                touched: true
              },
              myOtherField: {
                touched: true
              }
            }
          }
        }
      }), _extends({}, (0, _actions.untouch)('deep.myField', 'deep.myOtherField'), {
        form: 'foo'
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            deep: {
              myField: 'value',
              myOtherField: 'otherValue'
            }
          },
          fields: {
            deep: {
              myField: {},
              myOtherField: {}
            }
          }
        }
      });
    });

    it('should unmark array fields as touched on untouch', function () {
      var state = reducer(fromJS({
        foo: {
          values: {
            myFields: ['value', 'otherValue']
          },
          fields: {
            myFields: [{ touched: true }, { touched: true }]
          }
        }
      }), _extends({}, (0, _actions.untouch)('myFields[0]', 'myFields[1]'), {
        form: 'foo'
      }));
      expect(state).toEqualMap({
        foo: {
          values: {
            myFields: ['value', 'otherValue']
          },
          fields: {
            myFields: [{}, {}]
          }
        }
      });
    });
  };
};

exports.default = describeUntouch;