'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var isObject = function isObject(value) {
  return value && (typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object';
};

/**
 * Flattens a deep structure into a flat map with dot syntax keys. So:
 *
 * {
 *   a: {
 *     b: {
 *       c: 'the',
 *       d: 'quick'
 *     },
 *     c: 'brown'
 *   },
 *   d: [
 *     'fox',
 *     'jumps',
 *     {
 *       e: 'over'
 *     }
 *   ]
 * }
 *
 * flattens to:
 *
 * {
 *   'a.b.c': 'the',
 *   'a.b.d': 'quick',
 *   'a.c': 'brown',
 *   'd[0]': 'fox',
 *   'd[1]': 'jumps',
 *   'd[2].e': 'over'
 * }
 *
 * @param object The current value we are evaluating
 * @param path The path to here
 * @param accumulator The accumulator where the values go
 */
var flattenPaths = function flattenPaths(object) {
  var path = arguments.length <= 1 || arguments[1] === undefined ? '' : arguments[1];
  var accumulator = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  if (object) {
    if (Array.isArray(object)) {
      object.forEach(function (value, index) {
        flattenPaths(value, path ? path + '[' + index + ']' : String(index), accumulator);
      });
    } else if (isObject(object)) {
      Object.keys(object).forEach(function (key) {
        flattenPaths(object[key], path ? path + '.' + key : key, accumulator);
      });
    } else if (path) {
      accumulator[path] = object;
    }
  }
  return accumulator;
};

exports.default = flattenPaths;