'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var sameContents = function sameContents(arrayA, arrayB) {
  return arrayA === arrayB || arrayA.length === arrayB.length && arrayA.every(function (valueA, index) {
    return valueA === arrayB[index];
  });
};

var generate = function generate(field) {
  var tree = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var values = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];
  var path = arguments.length <= 3 || arguments[3] === undefined ? '' : arguments[3];
  var createField = arguments[4];
  var createArray = arguments[5];

  var _$exec = /([^.]+)\.?(.+)?/.exec(field);

  var _$exec2 = _slicedToArray(_$exec, 3);

  var all = _$exec2[0];
  var key = _$exec2[1];
  var rest = _$exec2[2]; // eslint-disable-line no-unused-vars

  if (/.+\[\]/.test(key)) {
    var _ret = function () {
      // is array key
      var noBrackets = key.substring(0, key.length - 2);
      var valuesArray = values[noBrackets];
      if (valuesArray && !Array.isArray(valuesArray)) {
        throw new Error('Expected array value for ' + path + field + ', but found ' + (typeof array === 'undefined' ? 'undefined' : _typeof(array)) + ': ' + JSON.stringify(valuesArray));
      }
      var treeArray = tree[noBrackets] || [];
      var results = [];
      if (valuesArray) {
        if (rest) {
          valuesArray.forEach(function (value, index) {
            results[index] = generate(rest, treeArray[index], value, '' + path + noBrackets + '[' + index + '].', createField, createArray);
          });
        } else {
          valuesArray.forEach(function (value, index) {
            var fieldKey = '' + path + noBrackets + '[' + index + ']';
            results[index] = !treeArray[index] || treeArray[index].key !== fieldKey ? createField(fieldKey) : treeArray[index];
          });
        }
      }
      var arrayKey = '' + path + noBrackets;
      if (!tree[noBrackets] || !sameContents(treeArray, results) || treeArray.key !== arrayKey) {
        return {
          v: _extends({}, tree, _defineProperty({}, noBrackets, createArray(arrayKey, results)))
        };
      }
    }();

    if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
  } else if (rest) {
    // need to recurse
    var result = generate(rest, tree[key], values[key], '' + path + key + '.', createField, createArray);
    if (!tree[key] || tree[key] !== result) {
      return _extends({}, tree, _defineProperty({}, key, result));
    }
  } else {
    // value key
    var fieldKey = '' + path + key;
    var existing = tree[key];
    if (!existing || existing.key !== fieldKey) {
      return _extends({}, tree, _defineProperty({}, key, createField(fieldKey)));
    }
  }
  return tree;
};

/**
 * Generates a tree of field objects
 *
 * @param fields The fields array passed to redux-form
 * @param tree The existing field tree
 * @param values The form values from the Redux store
 * @param createField A callback to create a field object
 * @param createArray A callback to create a field array
 */
var generateFields = function generateFields() {
  var fields = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];
  var tree = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
  var values = arguments[2];
  var createField = arguments[3];
  var createArray = arguments[4];
  return fields.reduce(function (accumulator, field) {
    return generate(field, accumulator, values, undefined, createField, createArray);
  }, tree);
};

exports.default = generateFields;