'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _actionTypes = require('./actionTypes');

var _lodash = require('lodash.forin');

var _lodash2 = _interopRequireDefault(_lodash);

var _lodash3 = require('lodash.mapvalues');

var _lodash4 = _interopRequireDefault(_lodash3);

var _deleteInWithCleanUp = require('./deleteInWithCleanUp');

var _deleteInWithCleanUp2 = _interopRequireDefault(_deleteInWithCleanUp);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var createReducer = function createReducer(structure) {
  var _behaviors;

  var empty = structure.empty;
  var getIn = structure.getIn;
  var setIn = structure.setIn;
  var deleteIn = structure.deleteIn;
  var fromJS = structure.fromJS;

  var deleteInWithCleanUp = (0, _deleteInWithCleanUp2.default)(structure);

  var behaviors = (_behaviors = {}, _defineProperty(_behaviors, _actionTypes.ADD_ARRAY_VALUE, function (state) {
    return state;
  }), _defineProperty(_behaviors, _actionTypes.BLUR, function (state, _ref, validate) {
    var field = _ref.field;
    var value = _ref.value;
    var touch = _ref.touch;

    var result = state;
    if (value !== undefined) {
      result = setIn(result, 'values.' + field, value);
    }
    if (value === '') {
      result = deleteInWithCleanUp(result, 'values.' + field);
    }
    result = deleteIn(result, 'active');
    result = deleteIn(result, 'fields.' + field + '.active');
    if (touch) {
      result = setIn(result, 'fields.' + field + '.touched', true);
    }
    return validate(result);
  }), _defineProperty(_behaviors, _actionTypes.CHANGE, function (state, _ref2, validate) {
    var field = _ref2.field;
    var value = _ref2.value;
    var touch = _ref2.touch;

    var result = state;
    if (value !== undefined) {
      result = setIn(result, 'values.' + field, value);
    }
    if (value === '') {
      result = deleteInWithCleanUp(result, 'values.' + field);
    }
    result = deleteInWithCleanUp(result, 'asyncErrors.' + field);
    result = deleteInWithCleanUp(result, 'submitErrors.' + field);
    if (touch) {
      result = setIn(result, 'fields.' + field + '.touched', true);
    }
    return validate(result);
  }), _defineProperty(_behaviors, _actionTypes.FOCUS, function (state, _ref3) {
    var field = _ref3.field;

    var result = state;
    var previouslyActive = getIn(state, 'active');
    result = deleteIn(result, 'fields.' + previouslyActive + '.active');
    result = setIn(result, 'fields.' + field + '.visited', true);
    result = setIn(result, 'fields.' + field + '.active', true);
    result = setIn(result, 'active', field);
    return result;
  }), _defineProperty(_behaviors, _actionTypes.INITIALIZE, function (state, _ref4) {
    var data = _ref4.data;

    var mapData = fromJS(data);
    var result = empty; // clean all field state
    result = setIn(result, 'values', mapData);
    result = setIn(result, 'initial', mapData);
    return result;
  }), _defineProperty(_behaviors, _actionTypes.REMOVE_ARRAY_VALUE, function (state) {
    return state;
  }), _defineProperty(_behaviors, _actionTypes.RESET, function (state, action, validate) {
    var values = getIn(state, 'initial');
    var result = empty;
    result = setIn(result, 'values', values);
    result = setIn(result, 'initial', values);
    return validate(result);
  }), _defineProperty(_behaviors, _actionTypes.START_ASYNC_VALIDATION, function (state, _ref5) {
    var field = _ref5.field;

    return setIn(state, 'asyncValidating', field || true);
  }), _defineProperty(_behaviors, _actionTypes.START_SUBMIT, function (state) {
    return setIn(state, 'submitting', true);
  }), _defineProperty(_behaviors, _actionTypes.STOP_ASYNC_VALIDATION, function (state, _ref6) {
    var errors = _ref6.errors;

    var result = state;
    result = deleteIn(result, 'asyncValidating');
    if (errors && Object.keys(errors).length) {
      var _error = errors._error;

      var fieldErrors = _objectWithoutProperties(errors, ['_error']);

      if (_error) {
        result = setIn(result, 'error', _error);
      }
      if (Object.keys(fieldErrors).length) {
        result = setIn(result, 'asyncErrors', fromJS(fieldErrors));
      } else {
        result = deleteIn(result, 'asyncErrors');
      }
    } else {
      result = deleteIn(result, 'error');
      result = deleteIn(result, 'asyncErrors');
    }
    return result;
  }), _defineProperty(_behaviors, _actionTypes.STOP_SUBMIT, function (state, _ref7) {
    var errors = _ref7.errors;

    var result = state;
    result = deleteIn(result, 'submitting');
    result = deleteIn(result, 'submitFailed');
    if (errors && Object.keys(errors).length) {
      var _error = errors._error;

      var fieldErrors = _objectWithoutProperties(errors, ['_error']);

      if (_error) {
        result = setIn(result, 'error', _error);
      }
      if (Object.keys(fieldErrors).length) {
        result = setIn(result, 'submitErrors', fromJS(fieldErrors));
      } else {
        result = deleteIn(result, 'submitErrors');
      }
      result = setIn(result, 'submitFailed', true);
    } else {
      result = deleteIn(result, 'error');
      result = deleteIn(result, 'submitErrors');
    }
    return result;
  }), _defineProperty(_behaviors, _actionTypes.SUBMIT_FAILED, function (state) {
    var result = state;
    result = setIn(result, 'submitFailed', true);
    result = deleteIn(result, 'submitting');
    return result;
  }), _defineProperty(_behaviors, _actionTypes.SWAP_ARRAY_VALUES, function (state) {
    return state;
  }), _defineProperty(_behaviors, _actionTypes.TOUCH, function (state, _ref8) {
    var fields = _ref8.fields;

    var result = state;
    fields.forEach(function (field) {
      return result = setIn(result, 'fields.' + field + '.touched', true);
    });
    return result;
  }), _defineProperty(_behaviors, _actionTypes.UNTOUCH, function (state, _ref9) {
    var fields = _ref9.fields;

    var result = state;
    fields.forEach(function (field) {
      return result = deleteIn(result, 'fields.' + field + '.touched');
    });
    return result;
  }), _behaviors);

  var makeValidator = function makeValidator(validate) {
    return function (state) {
      var result = state;
      var errors = validate(getIn(result, 'values') || empty);
      if (errors && Object.keys(errors).length) {
        var _error = errors._error;

        var fieldErrors = _objectWithoutProperties(errors, ['_error']);

        if (_error) {
          result = setIn(result, 'error', _error);
        }
        if (Object.keys(fieldErrors).length) {
          result = setIn(result, 'syncErrors', fromJS(fieldErrors));
        } else {
          result = deleteIn(result, 'syncErrors');
        }
      } else {
        result = deleteIn(result, 'error');
        result = deleteIn(result, 'syncErrors');
      }
      return result;
    };
  };

  var reducer = function reducer() {
    var state = arguments.length <= 0 || arguments[0] === undefined ? empty : arguments[0];
    var action = arguments[1];
    var validate = arguments.length <= 2 || arguments[2] === undefined ? function (state) {
      return state;
    } : arguments[2];

    var behavior = behaviors[action.type];
    return behavior ? behavior(state, action, validate) : state;
  };

  var initValidate = function initValidate(state, validation) {
    var result = state;
    (0, _lodash2.default)(validation, function (validate, form) {
      result = setIn(result, form, validate(getIn(result, form) || empty));
    });
    return result;
  };

  var byForm = function byForm(reducer) {
    var validation = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];
    return function () {
      var state = arguments.length <= 0 || arguments[0] === undefined ? initValidate(empty, validation) : arguments[0];
      var action = arguments[1];

      if (action === undefined) {
        return initValidate(state, validation);
      }
      var form = action.form;

      var rest = _objectWithoutProperties(action, ['form']); // eslint-disable-line no-redeclare


      if (!form) {
        return state;
      }
      if (action.type === _actionTypes.DESTROY) {
        return Object.keys(state).reduce(function (accumulator, formName) {
          return formName === form ? accumulator : _extends({}, accumulator, _defineProperty({}, formName, state[formName]));
        }, {});
      }
      var formState = getIn(state, form);
      var result = reducer(formState, rest, validation[form]);
      return result === formState ? state : setIn(state, form, result);
    };
  };

  /**
   * Adds additional functionality to the reducer
   */
  function decorate(target) {
    target.validation = function (validation) {
      return decorate(byForm(reducer, (0, _lodash4.default)(validation, makeValidator)));
    };

    // put back plugin and normalize?
    return target;
  }

  return decorate(byForm(reducer));
};

exports.default = createReducer;