'use strict';

var _deleteWithPath = require('../deleteWithPath');

var _deleteWithPath2 = _interopRequireDefault(_deleteWithPath);

var _immutable = require('immutable');

var _expectations = require('../expectations');

var _expectations2 = _interopRequireDefault(_expectations);

var _addExpectations = require('../../../__tests__/addExpectations');

var _addExpectations2 = _interopRequireDefault(_addExpectations);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('structure.immutable.deleteWithPath', function () {
  var expect = (0, _addExpectations2.default)(_expectations2.default);

  it('should not return state if path not found', function () {
    var state = (0, _immutable.fromJS)({ foo: 'bar' });
    expect((0, _deleteWithPath2.default)(state, /dog/)).toBe(state);
    expect((0, _deleteWithPath2.default)(state, /cat\.rat/)).toBe(state);
  });

  it('should delete shallow keys without mutating state', function () {
    var state = (0, _immutable.fromJS)({ foo: 'bar', dog: 'fido' });
    expect((0, _deleteWithPath2.default)(state, /foo/)).toNotBe(state).toEqualMap({ dog: 'fido' });
    expect((0, _deleteWithPath2.default)(state, /dog/)).toNotBe(state).toEqualMap({ foo: 'bar' });
  });

  it('should delete deep keys without mutating state', function () {
    var state = (0, _immutable.fromJS)({
      foo: {
        bar: ['baz', { dog: 42 }]
      },
      cat: {
        bar: 43,
        dog: true
      }
    });

    var result1 = (0, _deleteWithPath2.default)(state, /foo\.bar\[0\]/);
    expect(result1).toNotBe(state).toEqualMap({
      foo: {
        bar: [{ dog: 42 }]
      },
      cat: {
        bar: 43,
        dog: true
      }
    });
    expect(result1.get('foo')).toNotBe(state.get('foo'));
    expect(result1.get('foo').get('bar')).toNotBe(state.get('foo').get('bar'));
    expect(result1.get('foo').get('bar').count()).toBe(1);
    expect(result1.get('foo').get('bar').get(0)).toBe(state.get('foo').get('bar').get(1));
    expect(result1.get('cat')).toBe(state.get('cat'));

    var result2 = (0, _deleteWithPath2.default)(state, /.+bar\[1\]\.dog/);
    expect(result2).toNotBe(state).toEqualMap({
      foo: {
        bar: ['baz', {}]
      },
      cat: {
        bar: 43,
        dog: true
      }
    });
    expect(result2.get('foo')).toNotBe(state.get('foo'));
    expect(result2.get('foo').get('bar')).toNotBe(state.get('foo').get('bar'));
    expect(result2.get('foo').get('bar').get(0)).toBe(state.get('foo').get('bar').get(0));
    expect(result2.get('foo').get('bar').get(1)).toNotBe(state.get('foo').get('bar').get(1));
    expect(result2.get('cat')).toBe(state.get('cat'));

    var result3 = (0, _deleteWithPath2.default)(state, /.+\.dog$/);
    expect(result3).toNotBe(state).toEqualMap({
      foo: {
        bar: ['baz', {}]
      },
      cat: {
        bar: 43
      }
    });
    expect(result3.get('foo')).toNotBe(state.get('foo'));
    expect(result3.get('foo').get('bar')).toNotBe(state.get('foo').get('bar'));
    expect(result3.get('foo').get('bar').get(0)).toBe(state.get('foo').get('bar').get(0));
    expect(result3.get('foo').get('bar').get(1)).toNotBe(state.get('foo').get('bar').get(1));
    expect(result3.get('cat')).toNotBe(state.get('cat'));
    expect(result3.get('cat').get('bar')).toBe(state.get('cat').get('bar'));

    var result4 = (0, _deleteWithPath2.default)(state, /.+\.bar$/);
    expect(result4).toNotBe(state).toEqualMap({
      foo: {},
      cat: {
        dog: true
      }
    });
    expect(result4.get('foo')).toNotBe(state.get('foo'));
    expect(result4.get('cat')).toNotBe(state.get('cat'));
    expect(result4.get('cat').get('dog')).toBe(state.get('cat').get('dog'));
  });
});