'use strict';

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

var _deleteWithPath = require('../deleteWithPath');

var _deleteWithPath2 = _interopRequireDefault(_deleteWithPath);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('structure.plain.deleteWithPath', function () {
  it('should not return state if path not found', function () {
    var state = { foo: 'bar' };
    (0, _expect2.default)((0, _deleteWithPath2.default)(state, /dog/)).toBe(state);
    (0, _expect2.default)((0, _deleteWithPath2.default)(state, /cat\.rat/)).toBe(state);
  });

  it('should delete shallow keys without mutating state', function () {
    var state = { foo: 'bar', dog: 'fido' };
    (0, _expect2.default)((0, _deleteWithPath2.default)(state, /foo/)).toNotBe(state).toEqual({ dog: 'fido' });
    (0, _expect2.default)((0, _deleteWithPath2.default)(state, /dog/)).toNotBe(state).toEqual({ foo: 'bar' });
  });

  it('should delete deep keys without mutating state', function () {
    var state = {
      foo: {
        bar: ['baz', { dog: 42 }]
      },
      cat: {
        bar: 43,
        dog: true
      }
    };

    var result1 = (0, _deleteWithPath2.default)(state, /foo\.bar\[0\]/);
    (0, _expect2.default)(result1).toNotBe(state).toEqual({
      foo: {
        bar: [{ dog: 42 }]
      },
      cat: {
        bar: 43,
        dog: true
      }
    });
    (0, _expect2.default)(result1.foo).toNotBe(state.foo);
    (0, _expect2.default)(result1.foo.bar).toNotBe(state.foo.bar);
    (0, _expect2.default)(result1.foo.bar.length).toBe(1);
    (0, _expect2.default)(result1.foo.bar[0]).toBe(state.foo.bar[1]);
    (0, _expect2.default)(result1.cat).toBe(state.cat);

    var result2 = (0, _deleteWithPath2.default)(state, /.+bar\[1\]\.dog/);
    (0, _expect2.default)(result2).toNotBe(state).toEqual({
      foo: {
        bar: ['baz', {}]
      },
      cat: {
        bar: 43,
        dog: true
      }
    });
    (0, _expect2.default)(result2.foo).toNotBe(state.foo);
    (0, _expect2.default)(result2.foo.bar).toNotBe(state.foo.bar);
    (0, _expect2.default)(result2.foo.bar[0]).toBe(state.foo.bar[0]);
    (0, _expect2.default)(result2.foo.bar[1]).toNotBe(state.foo.bar[1]);
    (0, _expect2.default)(result2.cat).toBe(state.cat);

    var result3 = (0, _deleteWithPath2.default)(state, /.+\.dog$/);
    (0, _expect2.default)(result3).toNotBe(state).toEqual({
      foo: {
        bar: ['baz', {}]
      },
      cat: {
        bar: 43
      }
    });
    (0, _expect2.default)(result3.foo).toNotBe(state.foo);
    (0, _expect2.default)(result3.foo.bar).toNotBe(state.foo.bar);
    (0, _expect2.default)(result3.foo.bar[0]).toBe(state.foo.bar[0]);
    (0, _expect2.default)(result3.foo.bar[1]).toNotBe(state.foo.bar[1]);
    (0, _expect2.default)(result3.cat).toNotBe(state.cat);
    (0, _expect2.default)(result3.cat.bar).toBe(state.cat.bar);

    var result4 = (0, _deleteWithPath2.default)(state, /.+\.bar$/);
    (0, _expect2.default)(result4).toNotBe(state).toEqual({
      foo: {},
      cat: {
        dog: true
      }
    });
    (0, _expect2.default)(result4.foo).toNotBe(state.foo);
    (0, _expect2.default)(result4.cat).toNotBe(state.cat);
    (0, _expect2.default)(result4.cat.dog).toBe(state.cat.dog);
  });
});