'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _arguments = arguments;
/**
 * Mostly copied from https://github.com/substack/node-deep-equal
 *
 * The only change I've made, aside from making it ES6, was to force strict mode and make:
 *
 * deepEqual('', undefined) ==> true
 * deepEqual(undefined, '') ==> true
 *
 * This is necessary for an empty form, e.g. {}, to be pristine when all its values are ''
 */

var supportsArgumentsClass = function () {
  return Object.prototype.toString.call(_arguments);
}() == '[object Arguments]';

var isArguments = supportsArgumentsClass ? function (object) {
  return Object.prototype.toString.call(object) == '[object Arguments]';
} : function (object) {
  return object && (typeof object === 'undefined' ? 'undefined' : _typeof(object)) == 'object' && typeof object.length == 'number' && Object.prototype.hasOwnProperty.call(object, 'callee') && !Object.prototype.propertyIsEnumerable.call(object, 'callee') || false;
};

var pSlice = Array.prototype.slice;

var isUndefinedOrNull = function isUndefinedOrNull(value) {
  return value === null || value === undefined;
};

var isBuffer = function isBuffer(value) {
  return !(!value || (typeof value === 'undefined' ? 'undefined' : _typeof(value)) !== 'object' || typeof value.length !== 'number') && !(typeof value.copy !== 'function' || typeof value.slice !== 'function') && !(value.length > 0 && typeof value[0] !== 'number');
};

var objEquiv = function objEquiv(valueA, valueB) {
  if (isUndefinedOrNull(valueA) || isUndefinedOrNull(valueB)) return false;
  // an identical 'prototype' property.
  if (valueA.prototype !== valueB.prototype) return false;
  //~~~I've managed to break Object.keys through screwy arguments passing.
  //   Converting to array solves the problem.
  if (isArguments(valueA)) {
    if (!isArguments(valueB)) {
      return false;
    }
    valueA = pSlice.call(valueA);
    valueB = pSlice.call(valueB);
    return deepEqual(valueA, valueB);
  }
  if (isBuffer(valueA)) {
    if (!isBuffer(valueB)) return false;
    if (valueA.length !== valueB.length) return false;
    for (var i = 0; i < valueA.length; i++) {
      if (valueA[i] !== valueB[i]) return false;
    }
    return true;
  }
  var aKeys = void 0;
  var bKeys = void 0;
  try {
    aKeys = Object.keys(valueA);
    bKeys = Object.keys(valueB);
  } catch (e) {
    //happens when one is valueA string literal and the other isn't
    return false;
  }
  // having the same number of owned properties (keys incorporates
  // hasOwnProperty)
  if (aKeys.length != bKeys.length) return false;
  //the same set of keys (although not necessarily the same order),
  aKeys.sort();
  bKeys.sort();
  //~~~cheap key test
  for (var _i = aKeys.length - 1; _i >= 0; _i--) {
    if (aKeys[_i] != bKeys[_i]) return false;
  }
  //equivalent values for every corresponding key, and
  //~~~possibly expensive deep test
  for (var _i2 = aKeys.length - 1; _i2 >= 0; _i2--) {
    var key = aKeys[_i2];
    if (!deepEqual(valueA[key], valueB[key])) return false;
  }
  return (typeof valueA === 'undefined' ? 'undefined' : _typeof(valueA)) === (typeof valueB === 'undefined' ? 'undefined' : _typeof(valueB));
};

var deepEqual = function deepEqual(actual, expected) {
  // 7.1. All identical values are equivalent, as determined by ===.
  if (actual === expected) return true;

  if (actual === undefined && expected === '') return true;
  if (actual === '' && expected === undefined) return true;

  if (actual instanceof Date && expected instanceof Date) {
    return actual.getTime() === expected.getTime();
  }

  // 7.3. Other pairs that do not both pass typeof value == 'object',
  // equivalence is determined by ==.
  if (!actual || !expected || (typeof actual === 'undefined' ? 'undefined' : _typeof(actual)) != 'object' && (typeof expected === 'undefined' ? 'undefined' : _typeof(expected)) != 'object') {
    return actual === expected;
  }
  // 7.4. For all other Object pairs, including Array objects, equivalence is
  // determined by having the same number of owned properties (as verified
  // with Object.prototype.hasOwnProperty.call), the same set of keys
  // (although not necessarily the same order), equivalent values for every
  // corresponding key, and an identical 'prototype' property. Note: this
  // accounts for both named and indexed properties on Arrays.
  return objEquiv(actual, expected);
};

exports.default = deepEqual;